import logging
from typing import List
from datetime import datetime

from investing_algorithm_framework import PortfolioManager, Position, Order
from eltyer_investing_algorithm_framework.configuration import constants
from eltyer import Client

logger = logging.getLogger(__name__)


class EltyerPortfolioManager(PortfolioManager):
    identifier = "ELTYER"
    market = "ELTYER"
    client: Client = None

    def initialize(self, algorithm_context):
        self.client = algorithm_context.config.get(constants.ELTYER_CLIENT)
        portfolio = self.client.get_portfolio()
        self.market = portfolio.broker
        self.trading_symbol = portfolio.trading_symbol
        super(EltyerPortfolioManager, self).initialize(algorithm_context)

    def get_positions(
        self, algorithm_context=None, **kwargs
    ) -> List[Position]:
        positions_data = self.client.get_positions(json=True)
        positions = []

        for position_data in positions_data:
            positions.append(Position.from_dict(position_data))

        return positions

    def get_orders(
        self,
        symbol=None,
        since: datetime=None,
        algorithm_context=None,
        **kwargs
    ) -> List[Order]:
        orders_data = self.client.get_orders(json=True)
        orders = []

        for order_data in orders_data:
            orders.append(Order.from_dict(order_data))

        return orders



from eltyer_investing_algorithm_framework.configuration import constants
from eltyer_investing_algorithm_framework.setup import create_app
from eltyer_investing_algorithm_framework.initializer \
    import EltyerInitializer as Initializer
from eltyer_investing_algorithm_framework.portfolio_manager \
    import EltyerPortfolioManager as PortfolioManager
from eltyer_investing_algorithm_framework.order_executor \
    import EltyerOrderExecutor as OrderExecutor


__all__ = [
    "create_app",
    "Initializer",
    "PortfolioManager",
    "OrderExecutor",
    "constants"
]



import logging

from investing_algorithm_framework import OrderExecutor, Order, OrderStatus, \
    OrderType
from investing_algorithm_framework.core import OperationalException
from eltyer import Client, ClientException
from eltyer_investing_algorithm_framework.configuration import constants

logger = logging.getLogger(__name__)


class EltyerOrderExecutor(OrderExecutor):
    identifier = "ELTYER"

    def execute_order(
        self, order: Order, algorithm_context, **kwargs
    ) -> Order:
        client: Client = algorithm_context.config[constants.ELTYER_CLIENT]

        if OrderType.LIMIT.equals(order.type):

            try:
                eltyer_order = client.create_limit_order(
                    target_symbol=order.get_target_symbol(),
                    amount=order.get_amount_target_symbol(),
                    side=order.get_side(),
                    price=order.get_initial_price()
                )
                order.set_reference_id(eltyer_order.order_reference)
                order.set_status(OrderStatus.PENDING)
            except ClientException as e:
                logger.error(e)
                raise OperationalException(e)
        else:
            try:
                eltyer_order = client.create_market_order(
                    target_symbol=order.get_target_symbol(),
                    amount=order.get_amount_target_symbol()
                )
                order.set_reference_id(eltyer_order.order_reference)
                order.set_status(OrderStatus.PENDING)
            except ClientException as e:
                logger.exception(e)

        return order

    def check_order_status(
        self, order: Order, algorithm_context, **kwargs
    ) -> Order:
        client: Client = algorithm_context.config[constants.ELTYER_CLIENT]

        try:
            eltyer_order = client.get_order(
                reference_id=order.get_reference_id()
            )
            order.set_status(
                OrderStatus.from_string(eltyer_order.get_status())
            )
        except ClientException as e:
            logger.error(e)
            raise OperationalException(
                "Could not get order status from eltyer"
            )
        return order
